@@ -76,6 +76,7 @@ class AsyncProcess(object):
             raise ValueError(_('respawn_interval must be >= 0 if provided.'))
         self.respawn_interval = respawn_interval
         self._process = None
+        self._is_running = False
         self._kill_event = None
         self._reset_queues()
         self._watchers = []
@@ -104,10 +105,10 @@ class AsyncProcess(object):
         :raises eventlet.timeout.Timeout if blocking is True and the process
                 did not start in time.
         """
-        if self._kill_event:
+        LOG.debug('Launching async process [%s].', self.cmd)
+        if self._is_running:
             raise AsyncProcessException(_('Process is already started'))
         else:
-            LOG.debug('Launching async process [%s].', self.cmd)
             self._spawn()
 
         if block:
@@ -122,7 +123,7 @@ class AsyncProcess(object):
         :raises eventlet.timeout.Timeout if blocking is True and the process
                 did not stop in time.
         """
-        if self._kill_event:
+        if self._is_running:
             LOG.debug('Halting async process [%s].', self.cmd)
             self._kill(kill_signal)
         else:
@@ -133,6 +134,7 @@ class AsyncProcess(object):
 
     def _spawn(self):
         """Spawn a process and its watchers."""
+        self._is_running = True
         self._kill_event = eventlet.event.Event()
         self._process, cmd = utils.create_process(self._cmd,
                                                   run_as_root=self.run_as_root)
@@ -154,22 +156,16 @@ class AsyncProcess(object):
                 self._process.pid,
                 run_as_root=self.run_as_root)
 
-    def _kill(self, kill_signal, respawning=False):
-        """Kill the process and the associated watcher greenthreads.
-
-        :param respawning: Optional, whether respawn will be subsequently
-               attempted.
-        """
-        # Halt the greenthreads
-        self._kill_event.send()
-
+    def _kill(self, kill_signal):
+        """Kill the process and the associated watcher greenthreads."""
         pid = self.pid
         if pid:
+            self._is_running = False
             self._kill_process(pid, kill_signal)
 
-        if not respawning:
-            # Clear the kill event to ensure the process can be
-            # explicitly started again.
+        # Halt the greenthreads if they weren't already.
+        if self._kill_event:
+            self._kill_event.send()
             self._kill_event = None
 
     def _kill_process(self, pid, kill_signal):
@@ -194,15 +190,15 @@ class AsyncProcess(object):
         """Kill the async process and respawn if necessary."""
         LOG.debug('Halting async process [%s] in response to an error.',
                   self.cmd)
+        self._kill(signal.SIGKILL)
         if self.respawn_interval is not None and self.respawn_interval >= 0:
-            respawning = True
-        else:
-            respawning = False
-        self._kill(signal.SIGKILL, respawning=respawning)
-        if respawning:
             eventlet.sleep(self.respawn_interval)
             LOG.debug('Respawning async process [%s].', self.cmd)
-            self._spawn()
+            try:
+                self.start()
+            except AsyncProcessException:
+                # Process was already respawned by someone else...
+                pass
 
     def _watch_process(self, callback, kill_event):
         while not kill_event.ready():
@@ -217,10 +213,11 @@ class AsyncProcess(object):
             # Ensure that watching a process with lots of output does
             # not block execution of other greenthreads.
             eventlet.sleep()
-        # The kill event not being ready indicates that the loop was
+        # self._is_running being True indicates that the loop was
         # broken out of due to an error in the watched process rather
         # than the loop condition being satisfied.
-        if not kill_event.ready():
+        if self._is_running:
+            self._is_running = False
             self._handle_process_error()
 
     def _read(self, stream, queue):
