@@ -468,6 +468,30 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
                     context, agent_id, router_id)
 
 
+def _dvr_handle_unbound_allowed_addr_pair_add(
+        plugin, context, port, allowed_address_pair):
+    updated_port = plugin.update_unbound_allowed_address_pair_port_binding(
+        context, port, allowed_address_pair)
+    if updated_port:
+        LOG.debug("Allowed address pair port binding updated "
+                  "based on service port binding: %s", updated_port)
+        plugin.dvr_handle_new_service_port(context, updated_port)
+    plugin.update_arp_entry_for_dvr_service_port(context, port)
+
+
+def _dvr_handle_unbound_allowed_addr_pair_del(
+        plugin, context, port, allowed_address_pair):
+    updated_port = plugin.remove_unbound_allowed_address_pair_port_binding(
+        context, port, allowed_address_pair)
+    if updated_port:
+        LOG.debug("Allowed address pair port binding removed "
+                  "from service port binding: %s", updated_port)
+    aa_fixed_ips = plugin._get_allowed_address_pair_fixed_ips(context, port)
+    if aa_fixed_ips:
+        plugin.delete_arp_entry_for_dvr_service_port(
+            context, port, fixed_ips_to_delete=aa_fixed_ips)
+
+
 def _notify_l3_agent_new_port(resource, event, trigger, **kwargs):
     LOG.debug('Received %(resource)s %(event)s', {
         'resource': resource,
@@ -481,7 +505,7 @@ def _notify_l3_agent_new_port(resource, event, trigger, **kwargs):
             service_constants.L3_ROUTER_NAT)
         context = kwargs['context']
         l3plugin.dvr_handle_new_service_port(context, port)
-        l3plugin.update_arp_entry_for_dvr_service_port(context, port, "add")
+        l3plugin.update_arp_entry_for_dvr_service_port(context, port)
 
 
 def _notify_port_delete(event, resource, trigger, **kwargs):
@@ -490,7 +514,14 @@ def _notify_port_delete(event, resource, trigger, **kwargs):
     removed_routers = kwargs['removed_routers']
     l3plugin = manager.NeutronManager.get_service_plugins().get(
         service_constants.L3_ROUTER_NAT)
-    l3plugin.update_arp_entry_for_dvr_service_port(context, port, "del")
+    if port:
+        port_host = port.get(portbindings.HOST_ID)
+        allowed_address_pairs_list = port.get('allowed_address_pairs')
+        if allowed_address_pairs_list and port_host:
+            for address_pair in allowed_address_pairs_list:
+                _dvr_handle_unbound_allowed_addr_pair_del(
+                    l3plugin, context, port, address_pair)
+    l3plugin.delete_arp_entry_for_dvr_service_port(context, port)
     for router in removed_routers:
         # we need admin context in case a tenant removes the last dvr
         # serviceable port on a shared network owned by admin, where router
@@ -528,24 +559,64 @@ def _notify_l3_agent_port_update(resource, event, trigger, **kwargs):
                 }
                 _notify_port_delete(
                     event, resource, trigger, **removed_router_args)
+            fip = l3plugin._get_floatingip_on_port(context,
+                                                   port_id=original_port['id'])
+            if fip and not (removed_routers and
+                            fip['router_id'] in removed_routers):
+                l3plugin.l3_rpc_notifier.routers_updated_on_host(
+                    context, [fip['router_id']],
+                    original_port[portbindings.HOST_ID])
             if not n_utils.is_dvr_serviced(new_device_owner):
                 return
-        is_fixed_ips_changed = (
-            'fixed_ips' in new_port and
-            'fixed_ips' in original_port and
-            new_port['fixed_ips'] != original_port['fixed_ips'])
         is_new_port_binding_changed = (
             new_port[portbindings.HOST_ID] and
             (original_port[portbindings.HOST_ID] !=
                 new_port[portbindings.HOST_ID]))
+        new_port_profile = new_port.get(portbindings.PROFILE)
+        if new_port_profile:
+            # This check is required to prevent an arp update
+            # of the allowed_address_pair port.
+            if new_port_profile.get('original_owner'):
+                return
         if (is_new_port_binding_changed and
             n_utils.is_dvr_serviced(new_device_owner)):
             l3plugin.dvr_handle_new_service_port(context, new_port)
             l3plugin.update_arp_entry_for_dvr_service_port(
-                context, new_port, "add")
-        elif kwargs.get('mac_address_updated') or is_fixed_ips_changed:
+                context, new_port)
+            return
+        # Check for allowed_address_pairs and port state
+        new_port_host = new_port.get(portbindings.HOST_ID)
+        allowed_address_pairs_list = new_port.get('allowed_address_pairs')
+        if allowed_address_pairs_list and new_port_host:
+            new_port_state = new_port.get('admin_state_up')
+            original_port_state = original_port.get('admin_state_up')
+            if new_port_state and not original_port_state:
+                # Case were we activate the port from inactive state.
+                for address_pair in allowed_address_pairs_list:
+                    _dvr_handle_unbound_allowed_addr_pair_add(
+                        l3plugin, context, new_port, address_pair)
+                return
+            elif original_port_state and not new_port_state:
+                # Case were we deactivate the port from active state.
+                for address_pair in allowed_address_pairs_list:
+                    _dvr_handle_unbound_allowed_addr_pair_del(
+                        l3plugin, context, original_port, address_pair)
+                return
+            elif new_port_state and original_port_state:
+                # Case were the same port has additional address_pairs
+                # added.
+                for address_pair in allowed_address_pairs_list:
+                    _dvr_handle_unbound_allowed_addr_pair_add(
+                        l3plugin, context, new_port, address_pair)
+                return
+
+        is_fixed_ips_changed = (
+            'fixed_ips' in new_port and
+            'fixed_ips' in original_port and
+            new_port['fixed_ips'] != original_port['fixed_ips'])
+        if kwargs.get('mac_address_updated') or is_fixed_ips_changed:
             l3plugin.update_arp_entry_for_dvr_service_port(
-                context, new_port, "add")
+                context, new_port)
 
 
 def subscribe():
