@@ -343,7 +343,9 @@ class OVSNeutronAgent(sg_rpc.SecurityGroupAgentRpcCallbackMixin,
 
     def _restore_local_vlan_map(self):
         self._local_vlan_hints = {}
-        cur_ports = self.int_br.get_vif_ports()
+        # skip INVALID and UNASSIGNED to match scan_ports behavior
+        ofport_filter = (ovs_lib.INVALID_OFPORT, ovs_lib.UNASSIGNED_OFPORT)
+        cur_ports = self.int_br.get_vif_ports(ofport_filter)
         port_names = [p.port_name for p in cur_ports]
         port_info = self.int_br.get_ports_attributes(
             "Port", columns=["name", "other_config", "tag"], ports=port_names)
