@@ -24,6 +24,7 @@ import select
 import shlex
 import signal
 import subprocess
+import time
 
 import fixtures
 import netaddr
@@ -53,7 +54,7 @@ VETH1_PREFIX = 'test-veth1'
 PATCH_PREFIX = 'patch'
 
 SS_SOURCE_PORT_PATTERN = re.compile(
-    r'^.*\s+\d+\s+.*:(?P<port>\d+)\s+[0-9:].*')
+    r'^.*\s+\d+\s+.*:(?P<port>\d+)\s+[^\s]+:.*')
 
 READ_TIMEOUT = os.environ.get('OS_TEST_READ_TIMEOUT', 5)
 
@@ -85,12 +86,22 @@ def set_namespace_gateway(port_dev, gateway_ip):
     port_dev.route.add_gateway(gateway_ip)
 
 
-def assert_ping(src_namespace, dst_ip, timeout=1, count=1):
+def assert_ping(src_namespace, dst_ip, timeout=1, count=1, interval=1):
     ipversion = netaddr.IPAddress(dst_ip).version
     ping_command = 'ping' if ipversion == 4 else 'ping6'
     ns_ip_wrapper = ip_lib.IPWrapper(src_namespace)
-    ns_ip_wrapper.netns.execute([ping_command, '-c', count, '-W', timeout,
-                                 dst_ip])
+
+    # See bug 1588731 for explanation why using -c count ping option
+    # cannot be used and it needs to be done using the following workaround.
+    for _index in range(count):
+        start_time = time.time()
+        ns_ip_wrapper.netns.execute([ping_command, '-c', '1', '-W', timeout,
+                                     dst_ip])
+        end_time = time.time()
+        diff = end_time - start_time
+        if 0 < diff < interval:
+            # wait at most "interval" seconds between individual pings
+            time.sleep(interval - diff)
 
 
 @contextlib.contextmanager
@@ -145,7 +156,7 @@ def _get_source_ports_from_ss_output(output):
     for line in output.splitlines():
         match = SS_SOURCE_PORT_PATTERN.match(line)
         if match:
-            ports.add(match.group('port'))
+            ports.add(int(match.group('port')))
     return ports
 
 
