@@ -32,6 +32,7 @@ import webob.exc
 from neutron.agent.common import config as agent_config
 from neutron.agent.common import ovs_lib
 from neutron.agent.l3 import agent as neutron_l3_agent
+from neutron.agent.l3 import dvr_fip_ns
 from neutron.agent.l3 import dvr_snat_ns
 from neutron.agent.l3 import namespace_manager
 from neutron.agent.l3 import namespaces
@@ -341,6 +342,17 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
         ha_device = ip_lib.IPDevice(device_name, router.ns_name)
         ha_device.link.set_down()
 
+    def _assert_ping_reply_from_expected_address(
+        self, ping_result, expected_address):
+        ping_results = ping_result.split('\n')
+        self.assertGreater(
+            len(ping_results), 1,
+            "The result from ping should be multiple lines")
+        self.assertIn(
+            expected_address, ping_results[1],
+            ("Expect to see %s in the reply of ping, but failed" %
+             expected_address))
+
 
 class L3AgentTestCase(L3AgentTestFramework):
 
@@ -791,13 +803,12 @@ class L3AgentTestCase(L3AgentTestFramework):
                 router1.ns_name,
                 router1.get_ha_device_name()))
 
-    def test_fip_connection_from_same_subnet(self):
-        '''Test connection to floatingip which is associated with
-           fixed_ip on the same subnet of the source fixed_ip.
-           In other words it confirms that return packets surely
-           go through the router.
-        '''
-        router_info = self.generate_router_info(enable_ha=False)
+    def _setup_fip_with_fixed_ip_from_same_subnet(self, enable_snat):
+        """Setup 2 FakeMachines from same subnet, one with floatingip
+        associated.
+        """
+        router_info = self.generate_router_info(enable_ha=False,
+                                                enable_snat=enable_snat)
         router = self.manage_router(self.agent, router_info)
         router_ip_cidr = self._port_first_ip_cidr(router.internal_ports[0])
         router_ip = router_ip_cidr.partition('/')[0]
@@ -814,6 +825,16 @@ class L3AgentTestCase(L3AgentTestFramework):
         self._add_fip(router, dst_fip, fixed_address=dst_machine.ip)
         router.process(self.agent)
 
+        return src_machine, dst_machine, dst_fip
+
+    def test_fip_connection_from_same_subnet(self):
+        '''Test connection to floatingip which is associated with
+           fixed_ip on the same subnet of the source fixed_ip.
+           In other words it confirms that return packets surely
+           go through the router.
+        '''
+        src_machine, dst_machine, dst_fip = (
+            self._setup_fip_with_fixed_ip_from_same_subnet(enable_snat=True))
         protocol_port = net_helpers.get_free_namespace_port(
             l3_constants.PROTO_NAME_TCP, dst_machine.namespace)
         # client sends to fip
@@ -824,6 +845,16 @@ class L3AgentTestCase(L3AgentTestFramework):
         self.addCleanup(netcat.stop_processes)
         self.assertTrue(netcat.test_connectivity())
 
+    def test_ping_floatingip_reply_with_floatingip(self):
+        src_machine, _, dst_fip = (
+            self._setup_fip_with_fixed_ip_from_same_subnet(enable_snat=False))
+
+        # Verify that the ping replys with fip
+        ns_ip_wrapper = ip_lib.IPWrapper(src_machine.namespace)
+        result = ns_ip_wrapper.netns.execute(
+            ['ping', '-c', 1, '-W', 5, dst_fip])
+        self._assert_ping_reply_from_expected_address(result, dst_fip)
+
     def test_delete_external_gateway_on_standby_router(self):
         router_info = self.generate_router_info(enable_ha=True)
         router = self.manage_router(self.agent, router_info)
@@ -1105,6 +1136,69 @@ class TestDvrRouter(L3AgentTestFramework):
         self._assert_dvr_floating_ips(router)
         self._assert_snat_namespace_does_not_exist(router)
 
+    def test_dvr_router_fips_stale_gw_port(self):
+        self.agent.conf.agent_mode = 'dvr'
+
+        # Create the router with external net
+        dvr_router_kwargs = {'ip_address': '19.4.4.3',
+                             'subnet_cidr': '19.4.4.0/24',
+                             'gateway_ip': '19.4.4.1',
+                             'gateway_mac': 'ca:fe:de:ab:cd:ef'}
+        router_info = self.generate_dvr_router_info(**dvr_router_kwargs)
+        external_gw_port = router_info['gw_port']
+        ext_net_id = router_info['_floatingips'][0]['floating_network_id']
+        self.mock_plugin_api.get_external_network_id.return_value(ext_net_id)
+
+        # Create the fip namespace up front
+        stale_fip_ns = dvr_fip_ns.FipNamespace(ext_net_id,
+                                               self.agent.conf,
+                                               self.agent.driver,
+                                               self.agent.use_ipv6)
+        stale_fip_ns.create()
+
+        # Add a stale fg port to the namespace
+        fixed_ip = external_gw_port['fixed_ips'][0]
+        float_subnet = external_gw_port['subnets'][0]
+        fip_gw_port_ip = str(netaddr.IPAddress(fixed_ip['ip_address']) + 10)
+        prefixlen = netaddr.IPNetwork(float_subnet['cidr']).prefixlen
+        stale_agent_gw_port = {
+            'subnets': [{'cidr': float_subnet['cidr'],
+                         'gateway_ip': float_subnet['gateway_ip'],
+                         'id': fixed_ip['subnet_id']}],
+            'network_id': external_gw_port['network_id'],
+            'device_owner': l3_constants.DEVICE_OWNER_AGENT_GW,
+            'mac_address': 'fa:16:3e:80:8f:89',
+            'binding:host_id': self.agent.conf.host,
+            'fixed_ips': [{'subnet_id': fixed_ip['subnet_id'],
+                           'ip_address': fip_gw_port_ip,
+                           'prefixlen': prefixlen}],
+            'id': _uuid(),
+            'device_id': _uuid()}
+        stale_fip_ns.create_gateway_port(stale_agent_gw_port)
+
+        stale_dev_exists = self.device_exists_with_ips_and_mac(
+                stale_agent_gw_port,
+                stale_fip_ns.get_ext_device_name,
+                stale_fip_ns.get_name())
+        self.assertTrue(stale_dev_exists)
+
+        # Create the router, this shouldn't allow the duplicate port to stay
+        router = self.manage_router(self.agent, router_info)
+
+        # Assert the device no longer exists
+        stale_dev_exists = self.device_exists_with_ips_and_mac(
+                stale_agent_gw_port,
+                stale_fip_ns.get_ext_device_name,
+                stale_fip_ns.get_name())
+        self.assertFalse(stale_dev_exists)
+
+        # Validate things are looking good and clean up
+        self._validate_fips_for_external_network(
+            router, router.fip_ns.get_name())
+        ext_gateway_port = router_info['gw_port']
+        self._delete_router(self.agent, router.router_id)
+        self._assert_fip_namespace_deleted(ext_gateway_port)
+
     def test_dvr_update_floatingip_statuses(self):
         self.agent.conf.agent_mode = 'dvr'
         self._test_update_floatingip_statuses(self.generate_dvr_router_info())
@@ -1422,6 +1516,22 @@ class TestDvrRouter(L3AgentTestFramework):
             router_ns, floating_ips[0]['fixed_ip_address'])
         self.assertNotEqual(fip_rule_prio_1, fip_rule_prio_2)
 
+    def test_dvr_router_floating_ip_moved(self):
+        self.agent.conf.agent_mode = 'dvr'
+        router_info = self.generate_dvr_router_info()
+        router = self.manage_router(self.agent, router_info)
+        floating_ips = router.router[l3_constants.FLOATINGIP_KEY]
+        router_ns = router.ns_name
+        fixed_ip = floating_ips[0]['fixed_ip_address']
+        self.assertTrue(self._fixed_ip_rule_exists(router_ns, fixed_ip))
+        # Floating IP reassigned to another fixed IP
+        new_fixed_ip = '10.0.0.2'
+        self.assertNotEqual(new_fixed_ip, fixed_ip)
+        floating_ips[0]['fixed_ip_address'] = new_fixed_ip
+        self.agent._process_updated_router(router.router)
+        self.assertFalse(self._fixed_ip_rule_exists(router_ns, fixed_ip))
+        self.assertTrue(self._fixed_ip_rule_exists(router_ns, new_fixed_ip))
+
     def _get_fixed_ip_rule_priority(self, namespace, fip):
         iprule = ip_lib.IPRule(namespace)
         lines = iprule.rule._as_root([4], ['show']).splitlines()
@@ -1430,6 +1540,17 @@ class TestDvrRouter(L3AgentTestFramework):
                 info = iprule.rule._parse_line(4, line)
                 return info['priority']
 
+    def _fixed_ip_rule_exists(self, namespace, ip):
+        iprule = ip_lib.IPRule(namespace)
+        lines = iprule.rule._as_root([4], ['show']).splitlines()
+        for line in lines:
+            if ip in line:
+                info = iprule.rule._parse_line(4, line)
+                if info['from'] == ip:
+                    return True
+
+        return False
+
     def test_dvr_router_add_internal_network_set_arp_cache(self):
         # Check that, when the router is set up and there are
         # existing ports on the the uplinked subnet, the ARP
