@@ -435,6 +435,17 @@ class TestIpWrapper(base.BaseTestCase):
                 self.assertNotIn(mock.call().delete('ns'),
                                  ip_ns_cmd_cls.mock_calls)
 
+    def test_add_vlan(self):
+        retval = ip_lib.IPWrapper().add_vlan('eth0.1', 'eth0', '1')
+        self.assertIsInstance(retval, ip_lib.IPDevice)
+        self.assertEqual(retval.name, 'eth0.1')
+        self.execute.assert_called_once_with([], 'link',
+                                             ['add', 'link', 'eth0',
+                                              'name', 'eth0.1',
+                                              'type', 'vlan', 'id', '1'],
+                                             run_as_root=True, namespace=None,
+                                             log_fail_as_error=True)
+
     def test_add_vxlan_valid_port_length(self):
         retval = ip_lib.IPWrapper().add_vxlan('vxlan0', 'vni0',
                                               group='group0',
