@@ -251,7 +251,8 @@ class TunnelTest(object):
             mock.call.add_patch_port('patch-tun', 'patch-int'),
         ]
         self.mock_int_bridge_expected += [
-            mock.call.get_vif_ports(),
+            mock.call.get_vif_ports((ovs_lib.INVALID_OFPORT,
+                                     ovs_lib.UNASSIGNED_OFPORT)),
             mock.call.get_ports_attributes(
                 'Port', columns=['name', 'other_config', 'tag'], ports=[])
         ]
@@ -685,7 +686,8 @@ class TunnelTestUseVethInterco(TunnelTest):
             mock.call.add_patch_port('patch-tun', 'patch-int')
         ]
         self.mock_int_bridge_expected += [
-            mock.call.get_vif_ports(),
+            mock.call.get_vif_ports((ovs_lib.INVALID_OFPORT,
+                                     ovs_lib.UNASSIGNED_OFPORT)),
             mock.call.get_ports_attributes(
                 'Port', columns=['name', 'other_config', 'tag'], ports=[])
         ]
